#include <ddraw.h>
#include <dinput.h>
#include <dsound.h>
#include <dmksctrl.h>
#include <dmusici.h>
#include <dmusicc.h>
#include <dmusicf.h>

#ifndef DXLIB
#define DXLIB
// IntertainmenT Software - Copyright 1999
// DirectX (Game) Programming Library (for DirectX 6.1+)

// #defines
#define xRes 800
#define yRes 600
#define bpp 16

#define TRANSITION_BLUR	1
#define TRANSITION_CRUNCH	2
#define TRANSITION_FADETOBLACK	3
#define TRANSITION_FADETOWHITE	4
#define TRANSITION_DISSOLVE 5
#define TRANSITION_TELEPORT 6
#define TRANSITION_LIGHTENING 7

// macros
#define RGB216B(r,g,b) ((b&31) + ((g&63) << 5) + ((r&31) << 11))
#define GETR(colour) (colour>>11)
#define GETG(colour) ((colour>>5) & 0x3F)
#define GETB(colour) (colour & 0x1f)
// TYPES //////////////////////////////////////////////////

// PROTOTYPES /////////////////////////////////////////////

// EXTERNALS /////////////////////////////////////////////

extern HWND main_window_handle; // save the window handle
extern HINSTANCE main_instance; // save the instance

// Classes
class CParticle // Particle system
{
public:
	float x,y,dx,dy;
	int lifespan,size;
	USHORT color, endcolor, startcolor;
	bool fade_blink;	// true = fade   false = blink
	bool constant;
	bool alpha;			// true = alpha blending on
	bool vel_random;	// true = random movement   false = vel
	bool wrap;			// true = wrap around screen
	bool bounce;		// true = bounce off edges
	bool alive;			// true = particle is alive
	int rmax,gmax,bmax,rcounter,gcounter,bcounter;

	CParticle (void) { };
	CParticle (float x, float y, USHORT startcolor, USHORT endcolor, int lifespan, int size);
	CParticle (float x, float y, float dx, float dy, USHORT startcolor, USHORT endcolor, int lifespan, int size, bool fade_blink, bool alpha, bool vel_random, bool wrap, bool bounce, bool constant);

	void setupFade (void);
	void move ( void );
	void draw ( void );
};

class CMusic
{
public:
	IDirectMusicSegment			*dms;	// DirectMusic segment
	IDirectMusicSegmentState	*dmss;	// DirectMusic segment state
	int							state;	// state of MIDI

	CMusic ( void )
	{
		state = 0;
	}

	~CMusic ()
	{
		state = 0;
	}

	int LoadMIDI (char *filename);
	int Play ( void );
	int Stop ( void );
	bool isPlaying ( void );
	int Release ( void );
};

class CSound
{
public:
	LPDIRECTSOUNDBUFFER buffer;
	int	state;
	int rate;
	int size;

	int Setup ( int size, int flags );	// finito
	int Release ( void );		// finito
	int LoadSound ( char *name );  // finito
	int SetVolume ( int volume );  // finito
	int SetFrequency ( int freq ); // finito
	int SetPan ( int pan );		// finito
	int Play ( bool loop );		// finito
	int Stop ( void );			// finito
	int Pause ( void );			// finito
};

class CVector
{
public:
	int x,y;
	float mag;

	CVector ( void )
	{
		x=0;
		y=0;
		mag=0.0f;
	}
	CVector (int new_x, int new_y)
	{
		x=new_x;
		y=new_y;
		mag=(float)sqrt((double)(new_x*new_x) + (double)(new_y*new_y)); 
	}

	CVector operator+ (CVector a)
	{
		CVector b;
		b.x=x+a.x;
		b.y=y+a.y;
		return b;
	}
	CVector operator- (CVector a)
	{
		CVector b;
		b.x=x-a.x;
		b.y=y-a.y;
		return b;
	}
	void operator++ ( void )
	{
		x++;
		y++;
		updateMag ();
	}
	void operator-- ( void )
	{
		x--;
		y--;
		updateMag ();
	}
private:
	void updateMag ( void )
	{
		mag=(float)sqrt((double)(x*x) + (double)(y*y));
	}
};
// GLOBALS ////////////////////////////////////////////////

// sin and cos table
extern float sint[360];
extern float cost[360];

// DirectDraw
// notice that interface 4.0 is used on a number of interfaces
extern LPDIRECTDRAW7        lpdd;  // dd object
extern LPDIRECTDRAWSURFACE7 lpddsprimary;  // dd primary surface
extern LPDIRECTDRAWSURFACE7 lpddsback;  // dd back surface
extern LPDIRECTDRAWCLIPPER  lpddclipper;  // dd clipper
extern DDSURFACEDESC2       ddsd;                 // a direct draw surface description struct
extern USHORT               *primary_buffer; // primary video buffer
extern USHORT               *back_buffer; // secondary back buffer
extern int                  primary_lpitch;    // memory line pitch for primary buffer
extern int                  back_lpitch;    // memory line pitch for back buffer

// DirectInput
extern LPDIRECTINPUT lpdi;	// di object
extern LPDIRECTINPUTDEVICE lpdikey; // DirectInput keyboard device
extern LPDIRECTINPUTDEVICE lpdimouse; // DirectInput mouse device
extern UCHAR keystate[256];
extern DIMOUSESTATE mousestate;

// DirectSound
extern LPDIRECTSOUND lpds; // ds object
extern DSBUFFERDESC dsbd;	// ds buffer desc
extern WAVEFORMATEX pcmfx;	// holds format desc

// DirectMusic
extern IDirectMusicPerformance	*dmp;	// DirectMusic Performance
extern IDirectMusicLoader *dml;			// DirectMusic Loader

// Prototypes
void TableSetup (void);

// DIRECTMUSIC FUNCTIONS
int DMusic_Setup ( void );
int DMusic_Release ( void );

// DIRECTSOUND FUNCTIONS
int DSound_Setup ( void );
int DSound_Release ( void );

// DIRECTDRAW FUNCTIONS
// Graphic Primitives
int DDraw_fillSecondary (int x, int y, USHORT c, USHORT background);
int DDraw_drawLine(int x0, int y0, int x1, int y1, int color, USHORT *vb_start, int lpitch_2);
int DDraw_drawString (char *text, int x, int y, COLORREF color, LPDIRECTDRAWSURFACE7 surface);
int DDraw_fillRect (int x1, int y1, int x2, int y2, USHORT c, LPDIRECTDRAWSURFACE7 surface);

// Surface functions
int DDraw_Transition (int Transition);
USHORT *DDraw_LockSurface (LPDIRECTDRAWSURFACE7 surface, int *lPitch);
int DDraw_UnlockSurface (LPDIRECTDRAWSURFACE7 surface);
int DDraw_LoadBitmap(LPDIRECTDRAWSURFACE7 surface, LPCSTR szBitmap);
int DDraw_LoadGraphic (char *name, int length, LPDIRECTDRAWSURFACE7 surface);
int DDraw_SaveGraphic (char *name, LPDIRECTDRAWSURFACE7 surface);
int DDraw_Blit (int x, int y, int w, int h, LPDIRECTDRAWSURFACE7 from, LPDIRECTDRAWSURFACE7 to, bool transparent);
int DDraw_Blit (bool style, int x, int y, int u, int v, int w, int h, LPDIRECTDRAWSURFACE7 from, LPDIRECTDRAWSURFACE7 to, bool transparent);
LPDIRECTDRAWSURFACE7 DDraw_CreateTransparentSurface (int w, int h);
LPDIRECTDRAWSURFACE7 DDraw_CreateTransparentSurface (int w, int h, int flags);
LPDIRECTDRAWSURFACE7 DDraw_CreateSurface (int w, int h);
LPDIRECTDRAWSURFACE7 DDraw_CreateSurface (int w, int h, int flags);
int DDraw_AttachClipper (LPDIRECTDRAWSURFACE7 surface, int num_rects,LPRECT rect_list);
int DDraw_Flip ( void );
int DDraw_SecondaryLock ( void );
int DDraw_SecondaryUnlock ( void );
int DDraw_PrimaryLock ( void );
int DDraw_PrimaryUnlock ( void );
int DDraw_Setup ( void );
int DDraw_Release ( void );

// DIRECTINPUT FUNCTIONS
// Read functions
int DInput_KeyboardRead ( void );
int DInput_MouseRead ( void );

// Setup and initialization functions
int DInput_Setup ( void );
int DInput_KeyboardSetup ( void );
int DInput_MouseSetup ( void );
int DInput_Release ( void );
#endif